@@ -1,3 +1,4 @@
+###__PERLBIN__###
 #  Copyright (C) 2002-2004 Adrian Ulrich <pab at blinkenlights.ch>
 #  Part of the gnupod-tools collection
 #
@@ -28,19 +29,31 @@ use GNUpod::FooBar;
 use Getopt::Long;
 
 
-use vars qw($cid %pldb %spldb %itb %opts %meat %cmeat);
+use vars qw($cid %pldb %spldb %itb %opts %meat %cmeat @MPLcontent);
 #cid = CurrentID
 #pldb{name}  = array with id's
 #spldb{name} = '<spl' prefs
 #itb         = buffer
-$| = 1;
-print "mktunes.pl Version 0.94 (C) 2002-2004 Adrian Ulrich\n";
+#MPLcontent  = MasterPlaylist content (all songs)
+#              Note: if you don't add ALL songs to MPLcontent,
+#                    you'd break OTG and Rating AND the iPod
+#                    wouldn't boot if it finds a hidden-id in the
+#                    OTGPlaylist!!
 
+$| = 1;
+
+print "mktunes.pl ###__VERSION__### (C) Adrian Ulrich\n";
 
 $opts{mount} = $ENV{IPOD_MOUNTPOINT};
-GetOptions(\%opts, "help|h", "mount|m=s", "volume|v=i");
+
+GetOptions(\%opts, "version", "help|h", "ipod-name|n=s", "mount|m=s", "volume|v=i", "energy|e");
+GNUpod::FooBar::GetConfig(\%opts, {'ipod-name'=>'s', mount=>'s', volume=>'i', energy=>'b'}, "mktunes");
+
+$opts{'ipod-name'} ||= "GNUpod 0.95-20040531 aka rc1";
+
 
 usage() if $opts{help};
+version() if $opts{version};
 
 startup();
 
@@ -56,7 +69,7 @@ usage("$con->{status}\n") if $con->{status};
 print "! Volume-adjust set to $opts{volume} percent\n" if defined($opts{volume});
 
 print "> Parsing XML and creating FileDB\n";
-GNUpod::XMLhelper::doxml($con->{xml});
+GNUpod::XMLhelper::doxml($con->{xml}) or usage("Could not read $con->{xml}, did you run gnupod_INIT.pl ?");
 
 
 # Create header for mhits
@@ -106,6 +119,7 @@ close(ITB);
 
 print "> Updating Sync-Status\n";
 GNUpod::FooBar::setsync_itunesdb($con);
+GNUpod::FooBar::setvalid_otgdata($con);
 
 print "You can now umount your iPod. [Files: $itb{INFO}{FILES}]\n";
 print " - May the iPod be with you!\n\n";
@@ -117,14 +131,17 @@ print " - May the iPod be with you!\n\n";
 # Create a single playlist
 sub r_mpl {
  my($name, $type, $xidref, $spl) = @_;
+
 my $pl = undef;
 my $fc = 0;
 my $mhp = 0;
+
 if(ref($spl) eq "HASH") { #We got splpref!
  $pl .= GNUpod::iTunesDB::mk_splprefmhod({item=>$spl->{limititem},sort=>$spl->{limitsort},mos=>$spl->{moselected}
                                           ,liveupdate=>$spl->{liveupdate},value=>$spl->{limitval},
-                                          checkrule=>$spl->{checkrule}});
- $pl .= GNUpod::iTunesDB::mk_spldatamhod({anymatch=>$spl->{matchany},data=>$spldb{$name}});
+                                          checkrule=>$spl->{checkrule}}) || return undef;
+                                           
+ $pl .= GNUpod::iTunesDB::mk_spldatamhod({anymatch=>$spl->{matchany},data=>$spldb{$name}}) || return undef;
  $mhp=2; #Add a mhod
 }
 
@@ -147,19 +164,29 @@ if(ref($spl) eq "HASH") { #We got splpref!
 #########################################################################
 # Generate playlists from %pldb (+MPL)
 sub genpls {
- my @mpldat = (1..$cid);
- my ($pldata,undef) = r_mpl("GNUpod 0.94-20040229", 1,\@mpldat);
+
+ #Create mainPlaylist and set PlayListCount to 1
+ my ($pldata,undef) = r_mpl(Unicode::String::utf8($opts{'ipod-name'})->utf8, 1,\@MPLcontent);
  my $plc = 1;
+ 
 #CID is now used by r_mpl, dont use it yourself anymore
   foreach(GNUpod::XMLhelper::getpl_names()) {
-    my $splh = GNUpod::XMLhelper::get_splpref($_);
-    $plc++; #PLcount
-       my($pl, $xc) = r_mpl($_, 0, $pldb{$_}, $splh);
-       $pldata .= $pl;
-       my $plxt = "Smart-" if $splh;
-       print ">> Created $plxt"."Playlist '$_' with $xc file"; print "s" if $xc !=1;
-     
-     print "\n";
+    
+    my $splh = GNUpod::XMLhelper::get_splpref($_); #Get SPL Prefs
+    
+    my($pl, $xc) = r_mpl($_, 0, $pldb{$_}, $splh); #Kick Playlist creator
+    
+       if($pl) { #r_mpl got data, we can create a playlist..
+        $plc++;         #INC Playlist count
+        $pldata .= $pl; #Append data
+        #GUI Stuff
+        my $plxt = "Smart-" if $splh;
+        print ">> Created $plxt"."Playlist '$_' with $xc file"; print "s" if $xc !=1;
+        print "\n";
+       }
+       else {
+        warn "!! SKIPPED Playlist '$_', something went wrong...\n";
+       }     
   }
  
  return GNUpod::iTunesDB::mk_mhlp({playlists=>$plc}).$pldata;
@@ -173,12 +200,20 @@ sub build_mhit {
  my %chr = %{$xh};
  $chr{id} = $oid;
 my ($nhod,$cmhod,$cmhod_count) = undef;
+
  foreach(keys(%chr)) {
   next unless $chr{$_}; #Dont create empty fields
+
+  #Crop title if enabled
+  $chr{$_} = Unicode::String::utf8($chr{$_})->substr(0,18)->utf8 if $_ eq "title" && $opts{energy};
   $nhod = GNUpod::iTunesDB::mk_mhod({stype=>$_, string=>$chr{$_}});
+  next unless $nhod; #mk_mhod refused work, go to next item
   $cmhod .= $nhod;
-  $cmhod_count++ if defined $nhod;
+  $cmhod_count++;
  }
+ 
+  push(@MPLcontent,$oid);
+ 
      #Volume adjust
      if($opts{volume}) {
       $chr{volume} += int($opts{volume});
@@ -189,11 +224,12 @@ my ($nhod,$cmhod,$cmhod_count) = undef;
       }
      }
      
+     
      #Ok, we created the mhod's for this item, now we have to create an mhit
      my $mhit = GNUpod::iTunesDB::mk_mhit({size=>length($cmhod), count=>$cmhod_count, fh=>\%chr}).$cmhod;
      $itb{mhit}{_data_} .= $mhit;
      my $length = length($mhit);
-     $itb{INFO}{FILES}++;
+     $itb{INFO}{FILES}++; #Count all files (Needed for iTunesDB header (first part)
 
 return $length;
 }
@@ -244,10 +280,10 @@ sub xmk_newspl {
   push(@{$spldb{$name}}, $xr);
  }
 
- if(GNUpod::XMLhelper::get_splpref($name)->{liveupdate}) {
-  warn "mktunes.pl: warning: (pl: $name) Liveupdate enabled, but mktunes.pl doesn't support this atm. keeping songs\n";
+ unless(GNUpod::XMLhelper::get_splpref($name)->{liveupdate}) {
+  warn "mktunes.pl: warning: (pl: $name) Liveupdate disabled. Please set liveupdate=\"1\" if you don't want an empty playlist\n";
  }
- #elsif
+
  if(my $id = $el->{splcont}->{id}) { #We found an old id with disalbed liveupdate
     foreach(sort {$a <=> $b} split(/ /,$meat{id}{$id})) { push(@{$pldb{$name}}, $_); }
  }
@@ -310,10 +346,25 @@ die << "EOF";
 $rtxt
 Usage: mktunes.pl [-h] [-m directory] [-v VALUE]
 
-   -h, --help             : This ;)
-   -m, --mount=directory  : iPod mountpoint, default is \$IPOD_MOUNTPOINT
-   -v, --volume=VALUE     : Adjust volume +-VALUE% (example: -v -20)
+   -h, --help              display this help and exit
+       --version           output version information and exit
+   -m, --mount=directory   iPod mountpoint, default is \$IPOD_MOUNTPOINT
+   -n, --ipod-name=NAME    iPod Name (For unlabeled iPods)
+   -v, --volume=VALUE      Adjust volume +-VALUE% (example: -v -20)
                             (Works with Firmware 1.x and 2.x!)
+   -e, --energy            Save energy (= Disable scrolling title)
+
+Report bugs to <bug-gnupod\@nongnu.org>
+EOF
+}
+
+sub version {
+die << "EOF";
+mktunes.pl (gnupod) ###__VERSION__###
+Copyright (C) Adrian Ulrich 2002-2004
+
+This is free software; see the source for copying conditions.  There is NO
+warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 
 EOF
 }
